use fltk::{
	app,
	button::Button, 
	enums::CallbackTrigger,
	frame::Frame,
	input::*,
	prelude::*, 
	window::Window
};

#[derive (Clone, Copy)]
enum Message {
	OpenPort,
	ClosePort,
}

fn main () -> anyhow::Result <()> {
	tracing_subscriber::fmt::init ();
	let rt = tokio::runtime::Runtime::new ()?;
	
	let (fltk_tx, fltk_rx) = app::channel::<Message> ();
	
	let app = app::App::default();
	let mut wind = Window::new (100, 100, 800, 600, "PTTH client proxy");
	
	let margin = 10;
	let h = 30;
	let mut x = margin;
	let mut y = margin;
	
	let frame_status;
	
	{
		frame_status = Frame::new (10, 10, 800 - 20, 30, "0 ports forwarded");
	}
	
	y += h + margin;
	
	{
		let w = 80;
		let frame_client_port = Frame::new (x, y, w, h, "Local port");
		x += w + margin;
		
		let w = 120;
		let frame_server_id = Frame::new (x, y, w, h, "Server ID");
		x += w + margin;
		
		let w = 80;
		let frame_server_port = Frame::new (x, y, 80, h, "Server port");
		x += w + margin;
	}
	
	y += h + margin;
	
	{
		let w = 80;
		let mut input_client_port = Input::new (10, y, w, h, "");
		x += w + margin;
		
		let w = 120;
		let mut input_server_id = Input::new (100, y, w, h, "");
		x += w + margin;
		
		let w = 80;
		let mut input_server_port = Input::new (230, y, w, h, "");
		x += w + margin;
		
		let w = 80;
		let mut but_open = Button::new (320, y, w, h, "Open");
		x += w + margin;
		
		let w = 80;
		let mut but_close = Button::new (410, y, w, h, "Close");
		x += w + margin;
		
		input_client_port.set_value ("5901");
		input_server_id.set_value ("bogus_server");
		input_server_port.set_value ("5900");
		
		but_open.set_trigger (CallbackTrigger::Changed);
		but_open.emit (fltk_tx, Message::OpenPort);
		but_close.set_trigger (CallbackTrigger::Changed);
		but_close.emit (fltk_tx, Message::ClosePort);
		
		set_active (&mut but_open, true);
		set_active (&mut but_close, false);
	}
	
	y += h + margin;
	
	wind.end ();
	wind.show ();
	
	
	
	while app.wait () {
		match fltk_rx.recv () {
			Some (Message::OpenPort) => {
				
			},
			Some (Message::ClosePort) => {
				
			},
			None => (),
		}
	}
	
	Ok (())
}

fn set_active <W: WidgetExt> (w: &mut W, b: bool) {
	if b {
		w.activate ();
	}
	else {
		w.deactivate ();
	}
}
